/* eslint-env browser */
import AreaDeDesenho from './areaDesenho/AreaDeDesenho';
import MenuSuperior from './menus/MenuSuperior';

require('../scss/style.scss');

/**
 * Classe App
 *
 * Classe principal da aplicação. Realiza a instância dos elementos
 * que irão compor a aplicação
 *
 * @author Robertino Mendes Santiago Jr <robertino@ufpr.br>
 */
export default class App {
  /**
   * Método construtor da classe App.
   * São criados neste construtor:
   * . o menu superior
   * . a área de desenho
   */
  constructor() {
    this.menuSuperior = new MenuSuperior(this);
    this.areaDeDesenho = new AreaDeDesenho(this);
    this.objetos = [];
  }

  /**
     * Devolve o valor da propriedade menuSuperior
     * @return {MenuSuperior} Valor da propriedade menuSuperior
     */
  getMenuSuperior() {
    return this.menuSuperior;
  }

  setMenuSuperior(menuSuperior) {
    this.menuSuperior = menuSuperior;
  }

  /**
  * Devolve o valor da propriedade areaDeDesenho
  * @return {AreaDeDesenho} Valor da propriedade areaDeDesenho
  */
  getAreaDeDesenho() {
    return this.areaDeDesenho;
  }

  /**
  * Atribui um valor recebido por parâmetro à propriedade areaDeDesenho
  * @param {AreaDeDesenho} areaDeDesenho Valor da propriedade
  */
  setAreaDeDesenho(areaDeDesenho) {
    this.areaDeDesenho = areaDeDesenho;
  }

  /**
  * Devolve o valor da propriedade objetos
  * @return {Array} Valor da propriedade objetos
  */
  getObjetos() {
    return this.objetos;
  }

  /**
  * Atribui um valor recebido por parâmetro à propriedade objetos
  * @param {Array} objetos Objetos
  */
  setObjetos(objetos) {
    this.objetos = objetos;
  }
}

/**
 * Cria a instância da classe App após o carregamento da página
 */
document.addEventListener('DOMContentLoaded', () => {
  const app = new App(); // eslint-disable-line no-unused-vars
});
