import ObjetoDinamico from './ObjetoDinamico';
// eslint-disable-next-line import/no-cycle
import Ponto from './Ponto';

/**
 * Classe Rotulo
 *
 * Classe utilizada para gerar os rótulos (etiquetas) dos objetos geométricos
 *
 * @author Robertino Mendes Santiago Jr <robertino@ufpr.br>
 */
export default class Rotulo extends ObjetoDinamico {
  /**
   * Método construtor da classe
   */
  constructor(x, y, objeto) {
    super();
    this.gapX = 5;
    this.gapY = -5;
    this.x = x + this.gapX;
    this.y = y + this.gapY;
    this.objeto = objeto;
    this.valor = Rotulo.getRotulo(this.objeto);
    this.elemento = this.criaTexto();
  }

  /**
   * Método que devolve o elemento Rotulo.
   * @return {Object} Retorna o elemento que representa um Rótulo em formato SVG
   */
  draw() {
    return this.elemento;
  }

  /**
   * Método que cria um elemento SVG que representa um texto
   * @return {Object} Retorna o elemento SVG criado que representa o Rótulo
   */
  criaTexto() {
    const elemento = document.createElementNS(this.svgNS, 'text');
    elemento.setAttribute('x', this.x);
    elemento.setAttribute('y', this.y);
    elemento.setAttribute('fill', '#black');
    elemento.setAttribute('font-size', '10pt');
    elemento.textContent = this.valor;
    return elemento;
  }

  /**
   * Método estático que gera o rótulo para o objeto dinâmico
   * @param {ObjetoDinamico} objeto Objeto que receberá o rótulo
   * @return {String} Texto do rótulo
   */
  static getRotulo(objeto) {
    if (objeto instanceof Ponto) {
      const numero = Rotulo.getNumeroPonto();
      const letra = Rotulo.getLetraPonto();
      if (Rotulo.getLetraPonto() === 'Z') {
        Rotulo.setLetraPonto('A');
        Rotulo.setNumeroPonto(Rotulo.getNumeroPonto() + 1);
      } else {
        Rotulo.setLetraPonto(String.fromCharCode(Rotulo.getLetraPonto().charCodeAt(0) + 1));
      }
      return `${letra}${numero === 0 ? '' : numero}`;
    }
    return null;
  }

  /**
   * Método estático que retorna a letra atual utilizada para o rótulo de objeto do tipo Ponto
   * @return {String} letra atual utilizada para Pontos
   */
  static getLetraPonto() {
    return Rotulo.letraPonto;
  }

  /**
   * Atribui um valor recebido por parâmetro à propriedade estática letraPonto
   * @param {String} letra valor da letra
   */
  static setLetraPonto(letra) {
    Rotulo.letraPonto = letra;
  }

  /**
   * Método estático que retorna o número atual utilizado para o rótulo de objeto do tipo Ponto
   * @return {number} número atual utilizado para Pontos
   */
  static getNumeroPonto() {
    return Rotulo.numeroPonto;
  }

  /**
   * Atribui um valor recebido por parâmetro à propriedade estática numeroPonto
   * @param {number} numero valor do número
   */
  static setNumeroPonto(numero) {
    Rotulo.numeroPonto = numero;
  }
}

/**
 * Atributos estáticos utilizados para definir o valor do rótulo do Ponto
 */
Rotulo.letraPonto = 'A';
Rotulo.numeroPonto = 0;
